﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using MakeYourWeb.Models.DataModels;
using MakeYourWeb.Models;
using MakeYourWeb.Data;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Authentication;
using MakeYourWeb.Services;
using MakeYourWeb.Data.Abstract;
using MkwSharedLibrary;

namespace MakeYourWeb.Controllers
{
    [RequireHttps]
    [Produces("application/json")]
    [Route("api/[controller]/[action]")]
    public class AuthProviderController : AuthRelevantController
    {
        private readonly IAuthenticationRepository AuthRepository;



        public AuthProviderController(IAuthenticationRepository repo) : base()
        {
            AuthRepository = repo;
        }


        // POST: api/AuthProvider/SignIn
        [HttpPost]
        public IActionResult SignInUser([FromBody] SignInVM value)
        {
            try
            {
                string tokenValue = AuthRepository.SignInUser(value);
                if (tokenValue.IsNullOrEmpty())
                {
                    return Unauthorized();
                }

                return Json(tokenValue);
            }
            catch (Exception)
            {
                return new StatusCodeResult(StatusCodes.Status500InternalServerError);
            }
        }

        // POST: api/AuthProvider/RegisterUser
        [HttpPost]
        public IActionResult RegisterUser([FromBody] RegisterVM model)
        {
            try
            {
                int registerResult = AuthRepository.RegisterNewUser(model);
                var returnVal = registerResult > 0 ? (IActionResult)Ok() : StatusCode(StatusCodes.Status500InternalServerError);
                return returnVal;
            }
            catch (Exception exc)
            {
                return new StatusCodeResult(StatusCodes.Status500InternalServerError);
            }
        }

        // POST: api/AuthProvider/ResetPassword
        [HttpPost]
        public IActionResult ResetPassword([FromBody] ResetPasswordModel model)
        {
            try
            {
                if (base.IsAuthenticated)
                {
                    int response = AuthRepository.ChangePassword(base.UserIdentificator, model);

                    if (response > 0)
                        return Ok(response);
                    else
                        return BadRequest();
                }

                else return Unauthorized();
            }
            catch (Exception)
            {
                return new StatusCodeResult(StatusCodes.Status500InternalServerError);
            }
        }
    }
}
