﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using MakeYourWeb.Data;
using MakeYourWeb.Models;
using MakeYourWeb.Models.DataModels;
using MakeYourWeb.Data.Abstract;
using MakeYourWeb.Data.Repositories;
using Microsoft.AspNetCore.Authorization;
using MkwSharedLibrary;
using MkwSharedLibrary.Repositories;
using MkwSharedLibrary.Services;

namespace MakeYourWeb.Controllers
{
    [RequireHttps]
    [Produces("application/json")]
    [Route("api/[controller]/[action]/{id?}")]
    public class PageDataController : AuthRelevantController
    {
        private readonly IPageContentRepository PageDataBudgetRepository;
        private readonly ILogService Logger;

        public PageDataController(ApplicationDbContext context, ILogService logger) : base()
        {
            this.PageDataBudgetRepository = new PageContentRepository(context);
            this.Logger = logger;
        }

        // GET: api/PageData/HomePage
        [HttpGet]
        public async Task<IActionResult> HomePage()
        {
            try
            {
                HomePageDM resulContent = PageDataBudgetRepository.GetHomePageData();
                return Ok(resulContent);
            }
            catch (Exception exc)
            {
                int logresult = await Logger.SendLogAsync(new MkwSharedLibrary.Models.ServiceLogItem()
                {
                    LogType = MkwSharedLibrary.Models.LogType.Exception,
                    Message = exc.Message,
                    PriorityLevel = MkwSharedLibrary.Models.PriorityLevel.High,
                    SenderObject = exc.Source,
                    SenderSystemService = "MKW"
                });
                return new StatusCodeResult(StatusCodes.Status500InternalServerError);
            }
        }

        // GET: api/PageData/PortalGroupsPage
        [HttpGet]
        public async Task<IActionResult> PortalGroupsPage()
        {
            try
            {
                if (!base.IsAuthenticated)
                    return Unauthorized();

                var resulContent = await PageDataBudgetRepository.GetUsersPortalGroupsPageDataAsync(base.UserIdentificator);
                return Ok(resulContent);
            }
            catch (Exception exc)
            {
                int logresult = await Logger.SendLogAsync(new MkwSharedLibrary.Models.ServiceLogItem() {
                    LogType = MkwSharedLibrary.Models.LogType.Exception,
                    Message = exc.Message,
                    PriorityLevel = MkwSharedLibrary.Models.PriorityLevel.High,
                    SenderObject = exc.Source,
                    SenderSystemService = "MKW"
                });
                return new StatusCodeResult(StatusCodes.Status500InternalServerError);
            }
        }

        // GET: api/PageData/WelcomePage
        [HttpGet]
        public async Task<IActionResult> WelcomePage()
        {
            try
            {
                if (!base.IsAuthenticated)
                    return Unauthorized();

                WelcomePageDM resulContent = await PageDataBudgetRepository.GetWelcomePageDataAsync(base.UserIdentificator);
                return Ok(resulContent);
            }
            catch (Exception exc)
            {
                int logresult = await Logger.SendLogAsync(new MkwSharedLibrary.Models.ServiceLogItem()
                {
                    LogType = MkwSharedLibrary.Models.LogType.Exception,
                    Message = exc.Message,
                    PriorityLevel = MkwSharedLibrary.Models.PriorityLevel.High,
                    SenderObject = exc.Source,
                    SenderSystemService = "MKW"
                });
                return new StatusCodeResult(StatusCodes.Status500InternalServerError);
            }
        }

        // GET: api/PageData/SubscriptionsPage
        [HttpGet]
        public async Task<IActionResult> SubscriptionsPage()
        {
            try
            {
                if (!base.IsAuthenticated)
                    return Unauthorized();
                
                SubscriptionsPageDM resulContent = await PageDataBudgetRepository.GetSubscriptionsPageDataAsync(base.UserIdentificator, SubscribedChannels: true);
                return Ok(resulContent);
            }
            catch (Exception exc)
            {
                int logresult = await Logger.SendLogAsync(new MkwSharedLibrary.Models.ServiceLogItem()
                {
                    LogType = MkwSharedLibrary.Models.LogType.Exception,
                    Message = exc.Message,
                    PriorityLevel = MkwSharedLibrary.Models.PriorityLevel.High,
                    SenderObject = exc.Source,
                    SenderSystemService = "MKW"
                });
                return new StatusCodeResult(StatusCodes.Status500InternalServerError);
            }
        }

        // GET: api/PageData/RecommendedPage
        [HttpGet]
        public async Task<IActionResult> RecommendedPage()
        {
            try
            {
                if (!base.IsAuthenticated)
                    return Unauthorized();

                SubscriptionsPageDM resulContent = await PageDataBudgetRepository.GetSubscriptionsPageDataAsync(base.UserIdentificator, SubscribedChannels: false);
                return Ok(resulContent);
            }
            catch (Exception exc)
            {
                int logresult = await Logger.SendLogAsync(new MkwSharedLibrary.Models.ServiceLogItem()
                {
                    LogType = MkwSharedLibrary.Models.LogType.Exception,
                    Message = exc.Message,
                    PriorityLevel = MkwSharedLibrary.Models.PriorityLevel.High,
                    SenderObject = exc.Source,
                    SenderSystemService = "MKW"
                });
                return new StatusCodeResult(StatusCodes.Status500InternalServerError);
            }
        }

        // GET: api/PageData/ChannelDetailPage/{id}
        [HttpGet]
        public async Task<IActionResult> ChannelDetailPage(string id)
        {
            try
            {
                if (id.IsNullOrEmpty())
                    return BadRequest("No channel identificator.");

                ChannelDetailPageDM resulContent;
                if (this.UserIdentificator.IsNullOrEmpty())
                    resulContent = PageDataBudgetRepository.GetChannelDetailPageData(ChannelsIdentificator: id);
                else
                    resulContent = PageDataBudgetRepository.GetChannelDetailPageData(ChannelsIdentificator: id, UserIdentificator: this.UserIdentificator);

                return Ok(resulContent);
            }
            catch (Exception exc)
            {
                int logresult = await Logger.SendLogAsync(new MkwSharedLibrary.Models.ServiceLogItem()
                {
                    LogType = MkwSharedLibrary.Models.LogType.Exception,
                    Message = exc.Message,
                    PriorityLevel = MkwSharedLibrary.Models.PriorityLevel.High,
                    SenderObject = exc.Source,
                    SenderSystemService = "MKW"
                });

                return new StatusCodeResult(StatusCodes.Status500InternalServerError);
            }
        }

        // GET: api/PageData/UserAccountPage
        [HttpGet]
        public async Task<IActionResult> UserAccountPage()
        {
            try
            {
                if (!base.IsAuthenticated)
                      return Unauthorized();

                UserAccountPageDM result = PageDataBudgetRepository.GetUserAccountPageData(base.UserIdentificator);
                return Ok(result);
            }
            catch (Exception exc)
            {
                int logresult = await Logger.SendLogAsync(new MkwSharedLibrary.Models.ServiceLogItem()
                {
                    LogType = MkwSharedLibrary.Models.LogType.Exception,
                    Message = exc.Message,
                    PriorityLevel = MkwSharedLibrary.Models.PriorityLevel.High,
                    SenderObject = exc.Source,
                    SenderSystemService = "MKW"
                });

                return new StatusCodeResult(StatusCodes.Status500InternalServerError);
            }
        }

    }
}