﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using MakeYourWeb.Data.Abstract;
using MakeYourWeb.Models.DataModels;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using MakeYourWeb.Data;
using MakeYourWeb.Data.Repositories;
using MkwSharedLibrary.Models;
using MakeYourWeb.Services;

namespace MakeYourWeb.Controllers
{
    [RequireHttps]
    [Produces("application/json")]
    [Route("api/[controller]/[action]")]
    public class UsersWebsitesDataEditorController : AuthRelevantController
    {

        private readonly IServerDataEditor ServerDataEditorRepository;

        public UsersWebsitesDataEditorController(ApplicationDbContext context)
        {
            this.ServerDataEditorRepository = new ServerDataEditorRepository(context);
        }

        //POST: api/UsersWebsitesDataEditor/AddNewWebsite
        [HttpPost]
        public async Task<IActionResult> AddNewWebSite([FromBody] UrlPageModel model,[FromServices] IHtmlAnalyzeService HtmlService)
        {
            try
            {
                if (!base.IsAuthenticated)
                    return Unauthorized();

                await this.ServerDataEditorRepository.AddWebsiteSaveAsync(model, base.UserIdentificator, HtmlService);
                this.ServerDataEditorRepository.AddPost(model, base.UserIdentificator);
                int saveResult = await this.ServerDataEditorRepository.SaveDBAsync();

                return saveResult > 0 ? (IActionResult)Ok() : BadRequest();
            }
            catch (Exception)
            {
                return new StatusCodeResult(StatusCodes.Status500InternalServerError);
            }
        }

        //POST: api/UsersWebsitesDataEditor/AddNewWebSiteClick
        [HttpPost]
        public async Task<IActionResult> AddNewWebSiteClick([FromBody] UrlPageModel model)
        {
            try
            {
                if (!base.IsAuthenticated)
                    return Unauthorized();

                this.ServerDataEditorRepository.AddWebsiteClick(model, base.UserIdentificator);
                int result = await this.ServerDataEditorRepository.SaveDBAsync();
                return result > 0 ? (IActionResult)Ok() : BadRequest();
            }
            catch (Exception)
            {
                return new StatusCodeResult(StatusCodes.Status500InternalServerError);
            }
        }

        //POST: api/UsersWebsitesDataEditor/DeleteSite
        [HttpPost]
        public IActionResult DeleteSite([FromBody] UrlPageModel deleteModel)
        {
            try
            {
                if (!base.IsAuthenticated)
                    return Unauthorized();

                this.ServerDataEditorRepository.DeleteSavedSite(deleteModel, base.UserIdentificator);
                int result = this.ServerDataEditorRepository.SaveDB();

                return result > 0 ? (IActionResult)Ok() : BadRequest("No site deleted");
            }
            catch (Exception)
            {
                return new StatusCodeResult(StatusCodes.Status500InternalServerError);
            }
        }

        //POST: api/UsersWebsitesDataEditor/SubscribeChannel
        [HttpPost]
        public IActionResult SubscribeChannel([FromBody] SubscribeDM model)
        {
            try
            {
                if (!base.IsAuthenticated)
                    return Unauthorized();

                model.UserIdentificator = base.UserIdentificator;
                this.ServerDataEditorRepository.SubscribeChannel(model);
                int result = this.ServerDataEditorRepository.SaveDB();

                if (result > 0)
                    return new OkResult();

                else return new BadRequestResult();
            }
            catch (Exception)
            {
                return new StatusCodeResult(StatusCodes.Status500InternalServerError);
            }
        }

        //POST: api/UsersWebsitesDataEditor/UnsubscribeChannel
        [HttpPost]
        public IActionResult UnsubscribeChannel([FromBody] SubscribeDM model)
        {
            try
            {
                if (!base.IsAuthenticated)
                    return Unauthorized();

                model.UserIdentificator = base.UserIdentificator;
                this.ServerDataEditorRepository.DeleteSubscription(model);
                int result = this.ServerDataEditorRepository.SaveDB();

                if (result > 0)
                    return new OkResult();

                else return new BadRequestResult();
            }
            catch (Exception)
            {
                return new StatusCodeResult(StatusCodes.Status500InternalServerError);
            }
        }

        //POST: api/UsersWebsitesDataEditor/EditUserInfo
        [HttpPost]
        public IActionResult EditUserInfo([FromBody] UserInformationDM model)
        {
            try
            {
                if (!base.IsAuthenticated)
                    return Unauthorized();

                this.ServerDataEditorRepository.EditUserInformation(this.UserIdentificator, model);
                this.ServerDataEditorRepository.SaveDB();
                return new OkResult();
            }
            catch (Exception)
            {
                return new StatusCodeResult(StatusCodes.Status500InternalServerError);
            }
        }
    }
}

