﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.EntityFrameworkCore;
using MakeYourWeb.Models;
using MakeYourWeb.Services;

namespace MakeYourWeb.Data
{
    public class ApplicationDbContext : DbContext
    {
        public virtual DbSet<WebPortal> WebPortals { get; set; }
        public virtual DbSet<Website> Websites { get; set; }
        public virtual DbSet<AppUser> Users { get; set; }
        public virtual DbSet<SavedWebsite> SavedWebsites { get; set; }
        public virtual DbSet<WebsiteClick> WebsitesClicks { get; set; }
        public virtual DbSet<Post> Posts { get; set; }
        public virtual DbSet<Channel> UsersChannels { get; set; }
        public virtual DbSet<Subscription> Subscriptions { get; set; }

        public ApplicationDbContext(DbContextOptions<ApplicationDbContext> options) : base(options)
        {

        }

        public void InitializeTestingData()
        {
            //No users = no data => fill data
            if (!this.Users.Any())
                TestingInitializer.InitializeData(this);               
        }

        protected override void OnModelCreating(ModelBuilder modelBuilder)
        {
            //base.OnModelCreating(modelBuilder);

            //User + UserSecret
            modelBuilder.Entity<AppUser>()
                .HasOne(u => u.Secret)
                .WithOne(s => s.SecretOwner)
                .HasForeignKey<UserSecret>(u => u.UserID);

            //User-SitesConnectors 
            modelBuilder.Entity<AppUser>()
                .HasMany(u => u.SavedWebsites)
                .WithOne(s => s.User);

            modelBuilder.Entity<Website>()
                .HasMany(u => u.SiteSaves)
                .WithOne(s => s.WebSite);

            //Users-Sites Clicks
            modelBuilder.Entity<Website>()
            .HasMany(u => u.UsersClicks)
            .WithOne(s => s.ClickedSite);

            modelBuilder.Entity<AppUser>()
                .HasMany(u => u.UsersClicks)
                .WithOne(s => s.User);

            //Sites-Portal
            modelBuilder.Entity<WebPortal>()
                .HasMany(u => u.Sites)
                .WithOne(s => s.Portal);

            //Channel to user
            modelBuilder.Entity<AppUser>()
                .HasOne(u => u.UsersChannel)
                .WithOne(ch => ch.Owner)
                .HasForeignKey<Channel>(ch => ch.UserID);

            //Channel to posts
            modelBuilder.Entity<Channel>()
                .HasMany(ch => ch.PostsContent)
                .WithOne(p => p.ParentChannel);

            //Channel to subscriptions
            modelBuilder.Entity<Channel>()
                .HasMany(ch => ch.Subscriptions)
                .WithOne(s => s.SubscriptionChannel);

            //Post to website connectors
            modelBuilder.Entity<Website>()
                .HasMany(w => w.RelatedPosts)
                .WithOne(p => p.ConnectedWebsite);

            //User to subscriptions
            modelBuilder.Entity<AppUser>()
                .HasMany(u => u.UsersSubscriptions)
                .WithOne(s => s.Follower);

            //Auto generated channel string index value
            modelBuilder.Entity<Channel>()
                .Property(p => p.UniqueIdentificator)
                .IsRequired()
                .ValueGeneratedOnAdd();
        }

    }
}
