﻿using MakeYourWeb.Models;
using MakeYourWeb.Models.DataModels;
using MakeYourWeb.Services;
using Microsoft.AspNetCore.Authentication;
using Microsoft.AspNetCore.Authentication.JwtBearer;
using Microsoft.AspNetCore.Http;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Claims;
using System.Security.Cryptography;
using System.Text;
using System.Threading.Tasks;
using MkwSharedLibrary.Models;
using MakeYourWeb.Data.Abstract;
using MkwSharedLibrary;
using MkwSharedLibrary.Managers;
using Microsoft.EntityFrameworkCore;

namespace MakeYourWeb.Data.Repositories
{
    public class AuthRepository : IAuthenticationRepository
    {
        private readonly ApplicationDbContext DbContext;
        private readonly ISensitivePackage SensitivePackage;

        private const int TokenLiveTimeMinutes = 1000;


        public AuthRepository(ApplicationDbContext context, ISensitivePackage sensitive)
        {
            DbContext = context;
            this.SensitivePackage = sensitive;
        }

        public async Task LogOutAsync(string UserId, HttpContext HttpContext)
        {
            throw new NotImplementedException();
        }

        public int RegisterNewUser(RegisterVM Model)
        {
            if (IsEmailUsed(Model.Email))
                return -1;

            string InnerSalt = SecurityManager.GeneratePseudoRandomString(64/8);
            AppUser user = new AppUser()
            {
                Created = DateTime.Now,
                Email = Model.Email,
                Nick = Model.Nick,
                Description = Model.Description,
                SavedWebsites = new List<SavedWebsite>(),
                UsersChannel = new Channel() { UniqueIdentificator = InnerSalt },
                UsersClicks = new List<WebsiteClick>(),
                UsersSubscriptions = new List<Subscription>(),
                Secret = new UserSecret()
                {
                    EmailVirification = false,
                    Password = SecurityManager.MakePasswordHash(Model.Password, InnerSalt.ToByteArray()),
                    InnerId = InnerSalt
                }
            };

            DbContext.Users.Add(user);
            int saveResult = DbContext.SaveChanges();
            return saveResult;
        }

        public string SignInUser(SignInVM Model)
        {
            var match = DbContext.Users
                .FirstOrDefault(u => 
                u.Email.Equals(Model.Login) && 
                u.Secret.Password.Equals(SecurityManager.MakePasswordHash(Model.Password, u.Secret.InnerId.ToByteArray())));

            if (match.IsNotNull())
            {
                var user = match;            
                ITokenModel tokenModel = new TokenModel()
                {
                    ClientDomain = Model.Browser,
                    Created = DateTime.Now,
                    Expiration = DateTime.Now.AddMinutes(TokenLiveTimeMinutes),
                    UserIdentificator = user.UserID
                };

                return TokenManager.GenerateToken(SensitivePackage.GetSign(), tokenModel);
            }

            return String.Empty; //TODO Return value
        }

        private bool IsEmailUsed(string email) => this.DbContext.Users.Any(u => u.Email.Equals(email));

        public int ChangePassword(string UserIdentificator, ResetPasswordModel Model)
        {
            var user = DbContext.Users.Include(u=>u.Secret).FirstOrDefault(u => u.UserID.Equals(UserIdentificator));
            if (user != null)
            {
                var userSecret = user.Secret;
                bool passwordConfirm = Model.NewPassword.Equals(Model.NewPasswordConfirm);
                bool hashEqual = SecurityManager.MakePasswordHash(Model.OldPassword, userSecret.InnerId.ToByteArray()).Equals(userSecret.Password);

                if (passwordConfirm && hashEqual)
                {
                    userSecret.Password = SecurityManager.MakePasswordHash(Model.NewPassword, userSecret.InnerId.ToByteArray());
                    DbContext.Entry<UserSecret>(userSecret).State = Microsoft.EntityFrameworkCore.EntityState.Modified;
                }
                return DbContext.SaveChanges();
            }

            else return 0;
        }
    }
}
