﻿using MakeYourWeb.Data.Abstract;
using MakeYourWeb.Models;
using MakeYourWeb.Models.DataModels;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

namespace MakeYourWeb.Data.Repositories
{
    public class ChannelsRepository : BasicUserIncludedRepository, IChannelsRepository
    {
    

        public ChannelsRepository(ApplicationDbContext context) : base(context)
        {
        }

        public ChannelInformationDM GetLastSubscribedChannel(string UserIdentificator)
        {
            var subscription = DbContext
                .Subscriptions
                .Where(s => s.UserID.Equals(UserIdentificator))
                .Include(ch => ch.SubscriptionChannel)
                .FirstOrDefault();

            if (subscription != null)
            {
                Channel channel = subscription.SubscriptionChannel;
                AppUser owner = base.GetUser(channel.UserID, IncludeSecret: false);

                var channelDM = new ChannelInformationDM()
                {
                    ChannelIdentificator = channel.ChannelID,
                    Icon = channel.Owner.IconAvatar,
                    NumberOfPosts = DbContext.Posts.Count(p => p.ChannelID.Equals(channel.ChannelID)),
                    OwnerDescription = channel.Owner.Description,
                    OwnerNick = channel.Owner.Nick,
                    NumberOfFollowers = channel.Subscriptions.Count,
                    UniqueIdentificator = channel.UniqueIdentificator,
                    IsBlocked = channel.Blocked
                };
                return channelDM;
            }

            return null;
        }

        public IEnumerable<ChannelInformationDM> GetSubscribedChannels(string UsersIdentificator)
        {
            var followedChannels = DbContext.UsersChannels
                .Where(ch => ch.Subscriptions.Any(s => s.UserID.Equals(UsersIdentificator)));

            if (followedChannels.Any())
            {
                var channels = followedChannels
                    .Select(channel => new ChannelInformationDM()
                    {
                        ChannelIdentificator = channel.ChannelID,
                        Icon = channel.Owner.IconAvatar,
                        NumberOfPosts = channel.PostsContent.Count,
                        OwnerDescription = channel.Owner.Description,
                        OwnerNick = channel.Owner.Nick,
                        NumberOfFollowers = channel.Subscriptions.Count,
                        UniqueIdentificator = channel.UniqueIdentificator,
                        IsBlocked = channel.Blocked
                    })
                    .ToList();
                return channels;
            }
            else return new List<ChannelInformationDM>();
        }

        public IEnumerable<ChannelInformationDM> GetRecommendedChannels(string UserIdentificator)
        {
            var candidateChannels = DbContext.UsersChannels
                .Where(ch =>
                            !ch.UserID.Equals(UserIdentificator) &&
                            !ch.Subscriptions.Any(s => s.Follower.UserID.Equals(UserIdentificator)) &&
                            !ch.Blocked);

            if (!candidateChannels.Any())
                return new List<ChannelInformationDM>();

            return candidateChannels.Select(channel => new ChannelInformationDM()
            {
                ChannelIdentificator = channel.ChannelID,
                Icon = channel.Owner.IconAvatar,
                OwnerNick = channel.Owner.Nick,
                NumberOfPosts = channel.PostsContent.Count,
                OwnerDescription = channel.Owner.Description,
                NumberOfFollowers = channel.Subscriptions.Count,
                UniqueIdentificator = channel.UniqueIdentificator,
                IsBlocked = channel.Blocked
            })
               .ToList();
        }

        public IEnumerable<SubscriptionCardDM> GetRecommendedChannelsCards(string UserIdentificator)
        {
            var candidateChannels = DbContext.UsersChannels
                .Where(ch =>
                            !ch.UserID.Equals(UserIdentificator) &&
                            !ch.Subscriptions.Any(s => s.Follower.UserID.Equals(UserIdentificator)) &&
                            !ch.Blocked);

            if (!candidateChannels.Any())
                return new List<SubscriptionCardDM>();

            return candidateChannels.Select(channel => new SubscriptionCardDM()
            {
                ChannelIdentificator = channel.ChannelID,
                Icon = channel.Owner.IconAvatar,
                OwnerNick = channel.Owner.Nick,
                NumberOfPosts = channel.PostsContent.Count,
                OwnerDescription = channel.Owner.Description,
                NumberOfFollowers = channel.Subscriptions.Count,
                IsBlocked = channel.Blocked,
                UniqueIdentificator = channel.UniqueIdentificator,
                IsSubscribed = false
            })
               .ToList();
        }

        public IEnumerable<SubscriptionCardDM> GetSubscribedChannelsCards(string UsersIdentificator)
        {
            var followedChannels = DbContext.UsersChannels
                .Where(ch => ch.Subscriptions.Any(s => s.UserID.Equals(UsersIdentificator)));

            if (followedChannels.Any())
            {
                var channels = followedChannels
                    .Select(channel => new SubscriptionCardDM()
                    {
                        ChannelIdentificator = channel.ChannelID,
                        Icon = channel.Owner.IconAvatar,
                        NumberOfPosts = channel.PostsContent.Count,
                        OwnerDescription = channel.Owner.Description,
                        OwnerNick = channel.Owner.Nick,
                        NumberOfFollowers = channel.Subscriptions.Count,
                        IsBlocked = channel.Blocked,
                        UniqueIdentificator = channel.UniqueIdentificator,
                        IsSubscribed = true
                    })
                    .ToList();
                return channels;
            }
            else return new List<SubscriptionCardDM>();
        }

        public ChannelInformationDM GetChannelInformation(string ChannelIdentificator)
        {
            
            var channel =  DbContext.UsersChannels
                .Include(ch=>ch.Owner)
                .Include(ch=>ch.PostsContent)
                .Include(ch=>ch.Subscriptions)
                .First(ch => ch.UniqueIdentificator.Equals(ChannelIdentificator));

            return new ChannelInformationDM()
            {
                ChannelIdentificator = channel.ChannelID,
                Icon = channel.Owner.IconAvatar,
                IsBlocked = channel.Blocked,
                NumberOfFollowers = channel.Subscriptions.Count,
                NumberOfPosts = channel.PostsContent.Count,
                OwnerNick = channel.Owner.Nick,
                OwnerDescription = channel.Owner.Description,
                UniqueIdentificator = channel.UniqueIdentificator,
                IsSubscribed = false,
                IsUsersChannel = false
            };          
        }

        public ChannelInformationDM GetChannelInformation(string ChannelIdentificator, string UserIdentificator)
        {

            var channel = DbContext.UsersChannels
                .Include(ch => ch.Owner)
                .Include(ch => ch.PostsContent)
                .Include(ch => ch.Subscriptions)
                .First(ch => ch.UniqueIdentificator.Equals(ChannelIdentificator));

            return new ChannelInformationDM()
            {
                ChannelIdentificator = channel.ChannelID,
                Icon = channel.Owner.IconAvatar,
                IsBlocked = channel.Blocked,
                NumberOfFollowers = channel.Subscriptions.Count,
                NumberOfPosts = channel.PostsContent.Count,
                OwnerNick = channel.Owner.Nick,
                OwnerDescription = channel.Owner.Description,
                UniqueIdentificator = channel.UniqueIdentificator,
                IsSubscribed = DbContext.Subscriptions.Any(s => s.ChannelID.Equals(channel.ChannelID) && s.Follower.UserID.Equals(UserIdentificator)),
                IsUsersChannel = channel.UserID.Equals(UserIdentificator)
            };
        }

        public IEnumerable<WebCardComponentDM> GetChannelsPosts(string ChannelIdentificator)
        {
            IPostsRepository postsRepository = new PostsRepository(base.DbContext);
            var result = postsRepository.GetChannelsPosts(ChannelIdentificator);
            return result;
        }

        public IEnumerable<WebCardComponentDM> GetChannelsPosts(string ChannelIdentificator, string UserIdentificator)
        {
            IPostsRepository postsRepository = new PostsRepository(base.DbContext);
            var result = postsRepository.GetChannelsPosts(ChannelIdentificator, UserIdentificator);
            return result;
        }

        public ChannelInformationDM GetUsersChannelInformation(string UserIdentificator)
        {
            var channel =  DbContext.UsersChannels
                .Include(ch=>ch.Subscriptions)
                .Include(ch=>ch.PostsContent)
                .FirstOrDefault(ch => ch.UserID.Equals(UserIdentificator));

            return new ChannelInformationDM()
            {
                ChannelIdentificator = channel.ChannelID,
                Icon = channel.Owner.IconAvatar,
                IsBlocked = channel.Blocked,
                NumberOfFollowers = channel.Subscriptions.Count,
                NumberOfPosts = channel.PostsContent.Count,
                OwnerNick = channel.Owner.Nick,
                OwnerDescription = channel.Owner.Description,
                UniqueIdentificator = channel.UniqueIdentificator
            };
        }
    }
}
