﻿using MakeYourWeb.Data.Abstract;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using MakeYourWeb.Models.DataModels;
using MkwSharedLibrary.Models;

namespace MakeYourWeb.Data.Repositories
{
    public class PageContentRepository : IPageContentRepository
    {
        private readonly IWebsiteContentRepository UsersDataReader;
        private readonly IChannelsRepository ChannelsRepository;
        private readonly IPostsRepository PostsRepository;

        public PageContentRepository(ApplicationDbContext context)
        {
            this.UsersDataReader = new WebsiteContentRepository(context);
            this.ChannelsRepository = new ChannelsRepository(context);
            this.PostsRepository = new PostsRepository(context);
        }

        public ChannelDetailPageDM GetChannelDetailPageData(string ChannelsIdentificator)
        {
            var result = new ChannelDetailPageDM()
            {
                ChannelInformation = this.ChannelsRepository.GetChannelInformation(ChannelsIdentificator),
                ChannelPosts = this.ChannelsRepository.GetChannelsPosts(ChannelsIdentificator)
            };

            return result;
        }

        public ChannelDetailPageDM GetChannelDetailPageData(string ChannelsIdentificator, string UserIdentificator)
        {
            var result = new ChannelDetailPageDM()
            {
                ChannelInformation = this.ChannelsRepository.GetChannelInformation(ChannelsIdentificator, UserIdentificator),
                ChannelPosts = this.ChannelsRepository.GetChannelsPosts(ChannelsIdentificator, UserIdentificator)
            };

            return result;
        }

        public HomePageDM GetHomePageData()
        {
            return new HomePageDM()
            {
                LastPosts = this.PostsRepository.GetLastPosts(NumberFromTop: 3)
            };
        }

        public async Task<SubscriptionsPageDM> GetSubscriptionsPageDataAsync(string UserIdentificator, bool SubscribedChannels)
        {
            IEnumerable<SubscriptionCardDM> usersSubscriptions = SubscribedChannels ?
                ChannelsRepository.GetSubscribedChannelsCards(UserIdentificator) :
                ChannelsRepository.GetRecommendedChannelsCards(UserIdentificator);

            return new SubscriptionsPageDM()
            {
                SubscribtionContent = usersSubscriptions
            };
        }

        public UserAccountPageDM GetUserAccountPageData(string UsersIdentificator)
        {
            return new UserAccountPageDM() { UserInfo = this.UsersDataReader.GetUsersInfo(UsersIdentificator) };
        }

        public async Task<PortalGroupsPageDM> GetUsersPortalGroupsPageDataAsync(string UsersIdentificator)
        {
            return await this.UsersDataReader.GetUsersPortalGroupsAsync(UsersIdentificator);
        }



        public async Task<WelcomePageDM> GetWelcomePageDataAsync(string UserIdentificator)
        {

            int NumberOfSavedPortals = this.UsersDataReader.GetNumberOfUsersSavedPortals(UserIdentificator);
            int NumberOfSavedSites = this.UsersDataReader.GetNumberOfUsersSavedSites(UserIdentificator);
            string MostOpenedPortalName = this.UsersDataReader.GetUserFavoritePortalName(UserIdentificator);
            string MostOpenedSite = this.UsersDataReader.GetUserFavoriteWebsiteName(UserIdentificator);
            int NumberOfUsersPosts = this.PostsRepository.GetUsersPosts(UserIdentificator).Count();
            string LastPostTitle = this.PostsRepository.GetUsersPosts(UserIdentificator)?.OrderByDescending(p => p.Added).FirstOrDefault()?.MainHeader.MaxLenghtNormalize(100) ?? "";
            int NumberOfUsersFolowers = this.ChannelsRepository.GetUsersChannelInformation(UserIdentificator).NumberOfFollowers;

            UsersWebsitesStatsComponent UsersSitesStats = new UsersWebsitesStatsComponent()
            {
                NumberOfSavedPortals = NumberOfSavedPortals,
                NumberOfSavedSites = NumberOfSavedSites,
                MostOpenedPortalName = MostOpenedPortalName,
                MostOpenedSite = MostOpenedSite,
                NumberOfUsersPosts = NumberOfUsersPosts,
                LastPostTitle = LastPostTitle,
                NumberOfUsersFolowers = NumberOfUsersFolowers 
            };

            IEnumerable<WebSiteDM> LastOpenedSites = this.UsersDataReader.GetLastOpenedSitesData(UserIdentificator, HowManyTake: 10);
            IEnumerable<WebCardComponentDM> Webcards = await this.UsersDataReader.GetUsersWebcardsDataAsync(UserIdentificator, FromPostContent: true);
            IEnumerable<SubscriptionCardDM> SubscriptionsCards = this.ChannelsRepository.GetSubscribedChannelsCards(UserIdentificator);
            ChannelInformationDM LastOpenedChannel = ChannelsRepository.GetLastSubscribedChannel(UserIdentificator);
            IEnumerable<WebSiteDM> LastSavedSites = this.UsersDataReader.GetSavedWebsites(UserIdentificator, TakeFromTop: 10);
            UserInformationDM userInfo = this.UsersDataReader.GetUsersInfo(UserIdentificator);

            //Welcome page result
            return new WelcomePageDM()
            {
                LastOpenedSite = LastOpenedSites,
                WebCardsResource = Webcards,
                UsersContentStats = UsersSitesStats,
                UsersSubscriptions = new UsersSubscriptionsComponentDM()
                {
                    SubscribedChannelsCards = SubscriptionsCards,
                    LastAddedChannel = LastOpenedChannel
                },
                LastSavedSites = LastSavedSites,
                UserInfo = userInfo
            };
        }
    }
}
