﻿using MakeYourWeb.Data.Abstract;
using MakeYourWeb.Models.DataModels;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.EntityFrameworkCore;

namespace MakeYourWeb.Data.Repositories
{
    public class PostsRepository : BasicUserIncludedRepository, IPostsRepository
    {
        const int paragraphLenght = 200;

        public PostsRepository(ApplicationDbContext context) : base(context)
        {

        }

        public IEnumerable<WebCardComponentDM> GetLastPosts(int NumberFromTop)
        {
            if (DbContext.Posts.Any())
                return DbContext.Posts
                    .OrderByDescending(p => p.Created)
                    .Take(NumberFromTop)
                    .Select(p => new WebCardComponentDM()
                        {
                            Added = p.Created.ToShortDateString(),
                            Description = p.ConnectedWebsite.Description,
                            FirstImage = p.ConnectedWebsite.FirstImage,
                            HasOkStatus = p.ConnectedWebsite.HasOkStatusCode,
                            MainHeader = p.ConnectedWebsite.MainHeader,
                            MainParagraph = p.ConnectedWebsite.MainParagraph.MaxLenghtNormalize(paragraphLenght),
                            Title = p.ConnectedWebsite.Title,
                            Url = p.ConnectedWebsite.WebsiteURL,
                            WebPortalName = p.ConnectedWebsite.Portal.Name,
                            UserName = p.ParentChannel.Owner.Nick,
                            ChannelUniqueId = p.ParentChannel.UniqueIdentificator,
                            IsSaved = false
                        })
                        .ToList();

            else return new List<WebCardComponentDM>();
        }

        public IEnumerable<WebCardComponentDM> GetChannelsPosts(string ChannelIdentificator, string UserIdentificator)
        {
            var postCollection = DbContext.Posts
                .Include(p=>p.ParentChannel)
                .Include(p=>p.ConnectedWebsite)
                .Where(p => p.ParentChannel.UniqueIdentificator.Equals(ChannelIdentificator));

            if (postCollection.Any())
                return postCollection.OrderByDescending(ch => ch.Created).Select(p => new WebCardComponentDM()
                {
                    Added = p.Created.ToShortDateString(),
                    Description = p.ConnectedWebsite.Description,
                    FirstImage = p.ConnectedWebsite.FirstImage,
                    HasOkStatus = p.ConnectedWebsite.HasOkStatusCode,
                    MainHeader = p.ConnectedWebsite.MainHeader,
                    MainParagraph = p.ConnectedWebsite.MainParagraph.MaxLenghtNormalize(paragraphLenght),
                    Title = p.ConnectedWebsite.Title,
                    Url = p.ConnectedWebsite.WebsiteURL,
                    WebPortalName = p.ConnectedWebsite.Portal.Name,
                    UserName = p.ParentChannel.Owner.Nick,
                    ChannelUniqueId = p.ParentChannel.UniqueIdentificator,
                    IsSaved = DbContext.SavedWebsites.Any(s=>s.WebSiteURL.Equals(p.WebSiteURL) && s.UserID.Equals(UserIdentificator))
                }).ToList();

            return new List<WebCardComponentDM>();
        }

        public IEnumerable<WebCardComponentDM> GetChannelsPosts(string ChannelIdentificator)
        {
            var postCollection = DbContext.Posts
                .Include(p => p.ParentChannel)
                .Include(p => p.ParentChannel.Owner)
                .Include(p => p.ConnectedWebsite)
                .Where(p => p.ParentChannel.UniqueIdentificator.Equals(ChannelIdentificator));

            if (postCollection.Any())
                return postCollection
                    .OrderByDescending(ch => ch.Created)
                    .Select(p=>new WebCardComponentDM()
                            {
                                Added = p.Created.ToShortDateString(),
                                Description = p.ConnectedWebsite.Description,
                                FirstImage = p.ConnectedWebsite.FirstImage,
                                HasOkStatus = p.ConnectedWebsite.HasOkStatusCode,
                                MainHeader = p.ConnectedWebsite.MainHeader,
                                MainParagraph = p.ConnectedWebsite.MainParagraph.MaxLenghtNormalize(paragraphLenght),
                                Title = p.ConnectedWebsite.Title,
                                Url = p.ConnectedWebsite.WebsiteURL,
                                WebPortalName = p.ConnectedWebsite.Portal.Name,
                                UserName = p.ParentChannel.Owner.Nick,
                                ChannelUniqueId = p.ParentChannel.UniqueIdentificator,
                                IsSaved = false
                            })
                .ToList();

            return new List<WebCardComponentDM>();
        }

        public IEnumerable<WebCardComponentDM> GetUsersPosts(string UserIdentificator)
        {
            var postCollection = DbContext.Posts
                .Include(p => p.ParentChannel)
                .Include(p => p.ParentChannel.Owner)
                .Include(p => p.ConnectedWebsite)
                .Where(p => p.ParentChannel.Owner.UserID.Equals(UserIdentificator));

            if (postCollection.Any())
                return postCollection
                    .OrderByDescending(ch => ch.Created)
                    .Select(p => new WebCardComponentDM()
                    {
                        Added = p.Created.ToShortDateString(),
                        Description = p.ConnectedWebsite.Description,
                        FirstImage = p.ConnectedWebsite.FirstImage,
                        HasOkStatus = p.ConnectedWebsite.HasOkStatusCode,
                        MainHeader = p.ConnectedWebsite.MainHeader,
                        MainParagraph = p.ConnectedWebsite.MainParagraph.MaxLenghtNormalize(paragraphLenght),
                        Title = p.ConnectedWebsite.Title,
                        Url = p.ConnectedWebsite.WebsiteURL,
                        WebPortalName = p.ConnectedWebsite.Portal.Name,
                        UserName = p.ParentChannel.Owner.Nick,
                        ChannelUniqueId = p.ParentChannel.UniqueIdentificator,
                        IsSaved = false
                    })
                .ToList();

            return new List<WebCardComponentDM>();
        }
    }
}
