﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using MakeYourWeb.Models;
using MakeYourWeb.Models.DataModels;
using System.Net;
using Microsoft.EntityFrameworkCore;
using MakeYourWeb.Data.Abstract;
using MakeYourWeb.Data;
using MakeYourWeb.Data.Repositories;
using MkwSharedLibrary.Models;

namespace MakeYourWeb.Data.Repositories
{
    public class WebsiteContentRepository : BasicUserIncludedRepository, IWebsiteContentRepository
    {
        //values in characters
        const int MaxHeadLenght = 20;
        const int MaxParagraphLenght = 200;

        public WebsiteContentRepository(ApplicationDbContext context) : base(context)
        {
            //context binded inside parent - base constructor
        }

        public UserInformationDM GetUsersInfo(string UsersIdentificator)
        {
            var user = DbContext.Users.Include(u=>u.UsersChannel).FirstOrDefault(u => u.UserID.Equals(UsersIdentificator));

            return new UserInformationDM()
            {
                Description = user.Description,
                Icon = user.IconAvatar,
                Nick = user.Nick,
                Email = user.Email,
                Created = user.Created.ToShortDateString(),
                ChannelName = user.UsersChannel.UniqueIdentificator
            };
        }

        public int GetNumberOfUsersSavedSites(string UsersIdentificator) =>
            DbContext.SavedWebsites.Any() ?
            DbContext.SavedWebsites.Count(c => c.UserID.Equals(UsersIdentificator)) : 0;

        public int GetNumberOfUsersSavedPortals(string UsersIdentificator)
        {
            if (!DbContext.SavedWebsites.Any(c => c.UserID.Equals(UsersIdentificator)))
                return 0;

            return DbContext.SavedWebsites
                .Where(c => c.UserID.Equals(UsersIdentificator))
                .Select(selection => selection.WebSite.Portal)
                .GroupBy(g => g.PortalDomain)
                .Count();
        }

        public string GetUserFavoritePortalName(string UserIdentificator)
        {
            if (!DbContext.WebsitesClicks.Any(c => c.UserID.Equals(UserIdentificator)))
                return "No portal records";

            return DbContext.WebsitesClicks
                .Where(c => c.UserID.Equals(UserIdentificator))
                .GroupBy(g => g.ClickedSite.PortalDomain)
                ?.OrderByDescending(group => group.Count())
                .First().Key.ToString() ?? "No favorite portal";
        }

        public string GetUserFavoriteWebsiteName(string UserIdentificator)
        {
            if (!DbContext.WebsitesClicks.Any(c => c.UserID.Equals(UserIdentificator)))
                return "No websites saved.";

            return DbContext.WebsitesClicks
            .Where(c => c.UserID.Equals(UserIdentificator))
            .GroupBy(g => g.ClickedSite.MainHeader)
            .OrderByDescending(group => group.Count())
            .First().Key.ToString().MaxLenghtNormalize(MaxHeadLenght);
        }

        public IEnumerable<WebSiteDM> GetLastOpenedSitesData(string UserIdentificator, int HowManyTake)
        {
            if (!DbContext.WebsitesClicks.Any(c => c.UserID.Equals(UserIdentificator)))
                return new List<WebSiteDM>();

            var LastOpened = DbContext.WebsitesClicks
               .Where(c => c.UserID.Equals(UserIdentificator))
               .OrderByDescending(c => c.Created)
               .GroupBy(g => g.ClickedSite)
               .Take(HowManyTake)
               .OrderByDescending(group => group.First().Created)
               .Select(group => new WebSiteDM()
               {
                   Description = group.Key.Description,
                   FirstImage = group.Key.FirstImage,
                   HasOkStatus = group.Key.HasOkStatusCode,
                   MainHeader = group.Key.MainHeader,
                   MainParagraph = group.Key.MainParagraph,
                   Title = group.Key.Title,
                   Url = group.Key.WebsiteURL
               })
               .ToList();

            return LastOpened;
        }



        public async Task<IEnumerable<WebCardComponentDM>> GetUsersWebcardsDataAsync(string UserIdentificator, bool FromPostContent)
        {
            if (DbContext.WebPortals.Any() && DbContext.Websites.Any())
            {

                if (!FromPostContent)
                {
                    var savedSites = DbContext.SavedWebsites
                                   .Where(savedWebsite => savedWebsite.UserID.Equals(UserIdentificator));


                    var usersSavedSitesResult = savedSites.Any() ?
                                await DbContext.SavedWebsites
                                .Where(savedWebsite => savedWebsite.UserID.Equals(UserIdentificator))
                                .OrderByDescending(connector => connector.ConnectorCreated)
                                .Select(savedWebsite => new WebCardComponentDM()
                                {
                                    Added = savedWebsite.ConnectorCreated.ToShortDateString(),
                                    Description = savedWebsite.WebSite.Description,
                                    FirstImage = savedWebsite.WebSite.FirstImage,
                                    MainHeader = savedWebsite.WebSite.MainHeader.MaxLenghtNormalize(MaxHeadLenght),
                                    MainParagraph = savedWebsite.WebSite.MainParagraph.MaxLenghtNormalize(MaxParagraphLenght),
                                    Title = savedWebsite.WebSite.Title.MaxLenghtNormalize(MaxHeadLenght),
                                    Url = savedWebsite.WebSite.WebsiteURL,
                                    WebPortalName = savedWebsite.WebSite.Portal.Name.ToUpper(),
                                    HasOkStatus = savedWebsite.WebSite.HasOkStatusCode,
                                    UserName = savedWebsite.User.Nick,
                                    ChannelUniqueId = savedWebsite.User.UsersChannel.UniqueIdentificator,
                                    IsSaved = true
                                })
                                .ToListAsync() : new List<WebCardComponentDM>();

                    return usersSavedSitesResult;
                }

                else
                {
                    IQueryable<Post> userRelatedPosts = DbContext.Posts
                        .Where(postProps =>
                                postProps.ParentChannel.Subscriptions.Any(subscription => subscription.UserID.Equals(UserIdentificator)) ||
                                postProps.ParentChannel.UserID.Equals(UserIdentificator));


                    var postsResult = userRelatedPosts.Any() ?
                        await userRelatedPosts
                        .OrderByDescending(post => post.Created)
                        .Select(post => new WebCardComponentDM()
                        {
                            Added = post.Created.ToShortDateString(),
                            Description = post.ConnectedWebsite.Description,
                            FirstImage = post.ConnectedWebsite.FirstImage,
                            HasOkStatus = post.ConnectedWebsite.HasOkStatusCode,
                            MainHeader = post.ConnectedWebsite.MainHeader.MaxLenghtNormalize(MaxHeadLenght),
                            MainParagraph = post.ConnectedWebsite.MainParagraph.MaxLenghtNormalize(MaxParagraphLenght),
                            Title = post.ConnectedWebsite.Title.MaxLenghtNormalize(MaxHeadLenght),
                            Url = post.ConnectedWebsite.WebsiteURL,
                            UserName = post.ParentChannel.Owner.Nick,
                            WebPortalName = post.ConnectedWebsite.Portal.PortalDomain.ToUpper(),
                            ChannelUniqueId = post.ParentChannel.UniqueIdentificator,
                            IsSaved = DbContext.SavedWebsites.Any(savedSite=>savedSite.UserID.Equals(UserIdentificator) && savedSite.WebSiteURL.Equals(post.WebSiteURL))
                        })
                        .ToListAsync() : new List<WebCardComponentDM>();

                    return postsResult;
                }
            }

            return new List<WebCardComponentDM>(); //fresh empty DB
        }

        public async Task<PortalGroupsPageDM> GetUsersPortalGroupsAsync(string UsersIdentificator)
        {
            if (DbContext.WebPortals.Any() && DbContext.Websites.Any() && DbContext.Websites.Any(s => s.SiteSaves.Any(u => u.UserID.Equals(UsersIdentificator))))
            {
                IEnumerable<PortalGroupComponentDM> groups = DbContext
                    .Websites
                    .Include(s => s.SiteSaves)
                    .Where(s => s.SiteSaves.Where(c => c.UserID.Equals(UsersIdentificator)).Any())
                    .GroupBy(g => g.Portal)
                    .Select(group => new PortalGroupComponentDM()
                    {
                        Domain = group.Key.PortalDomain.ToUpper(),
                        Icon = group.Key.Image,
                        Name = group.Key.Name.ToUpper(),
                        WebSites = group
                        .OrderByDescending(s => s.AddedDate)
                        .Select(site => new WebSiteDM()
                        {
                            Added = site.SiteSaves.First(c => c.UserID.Equals(UsersIdentificator)).ConnectorCreated.ToShortDateString(),
                            Description = site.Description,
                            Title = site.Title.MaxLenghtNormalize(MaxHeadLenght),
                            Url = site.WebsiteURL,
                            MainHeader = site.MainHeader.MaxLenghtNormalize(MaxHeadLenght),
                            MainParagraph = site.MainParagraph.MaxLenghtNormalize(MaxParagraphLenght),
                            FirstImage = site.FirstImage,
                            HasOkStatus = site.HasOkStatusCode
                        })
                    })
                    .ToList<PortalGroupComponentDM>();

                return new PortalGroupsPageDM() { PortalGroups = groups };
            }

            return new PortalGroupsPageDM();
        }

        public IEnumerable<WebSiteDM> GetSavedWebsites(string UsersIdentificator, int? TakeFromTop)
        {
            if(TakeFromTop.HasValue)
                return DbContext.SavedWebsites.Where(w => w.UserID.Equals(UsersIdentificator))?.
                    OrderByDescending(w => w.ConnectorCreated)
                    .Take(TakeFromTop.Value)
                    .Select(w => new WebSiteDM()
                        {
                            Added = w.ConnectorCreated.ToShortDateString(),
                            Description = w.WebSite.Description,
                            FirstImage = w.WebSite.FirstImage,
                            HasOkStatus = w.WebSite.HasOkStatusCode,
                            MainHeader = w.WebSite.MainHeader,
                            MainParagraph = w.WebSite.MainParagraph,
                            Title = w.WebSite.Title,
                            Url = w.WebSiteURL,
                            WebPortalName = w.WebSite.Portal.Name
                        }).ToList();
            else
                return DbContext.SavedWebsites.Where(w => w.UserID.Equals(UsersIdentificator))?.
                            OrderByDescending(w => w.ConnectorCreated)
                            .Select(w => new WebSiteDM()
                            {
                                Added = w.ConnectorCreated.ToShortDateString(),
                                Description = w.WebSite.Description,
                                FirstImage = w.WebSite.FirstImage,
                                HasOkStatus = w.WebSite.HasOkStatusCode,
                                MainHeader = w.WebSite.MainHeader,
                                MainParagraph = w.WebSite.MainParagraph,
                                Title = w.WebSite.Title,
                                Url = w.WebSiteURL,
                                WebPortalName = w.WebSite.Portal.Name
                            }).ToList();
        }

    }
}
