﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Http;
using MakeYourWeb.Data;
using MakeYourWeb.Services;

namespace MakeYourWeb.Middlewares
{
    // You may need to install the Microsoft.AspNetCore.Http.Abstractions package into your project
    public class TestingDbFillMiddleware
    {
        private readonly RequestDelegate _next;

        public TestingDbFillMiddleware(RequestDelegate next)
        {
            _next = next;
        }

        public Task Invoke(HttpContext httpContext)
        {
            try
            {
                ApplicationDbContext DbContext = httpContext.RequestServices.GetService(typeof(ApplicationDbContext)) as ApplicationDbContext;
                ISensitivePackage sensitive = httpContext.RequestServices.GetService(typeof(ISensitivePackage)) as ISensitivePackage;

                DbContext.InitializeTestingData();

                return _next(httpContext);
            }
            catch (Exception)
            {
                httpContext.Response.Clear();
                httpContext.Response.StatusCode = StatusCodes.Status400BadRequest;
                return _next(httpContext);
            }
        }
    }

    // Extension method used to add the middleware to the HTTP request pipeline.
    public static class TestingDbFillMiddlewareExtensions
    {
        public static IApplicationBuilder UseTestingDbFillMiddleware(this IApplicationBuilder builder)
        {
            return builder.UseMiddleware<TestingDbFillMiddleware>();
        }
    }
}
