﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Http;
using System.Security.Claims;
using MakeYourWeb.Models.DataModels;
using MkwSharedLibrary.Models;
using MkwSharedLibrary.Managers;
using MakeYourWeb.Data;
using MakeYourWeb.Services;
using Microsoft.Extensions.Logging;
using MkwSharedLibrary.Services;

namespace MakeYourWeb.Middlewares
{
    // You may need to install the Microsoft.AspNetCore.Http.Abstractions package into your project
    public class TokenAuthMiddleware
    {
        private readonly RequestDelegate _next;

        public TokenAuthMiddleware(RequestDelegate next)
        {
            _next = next;
        }

        public async Task InvokeAsync(HttpContext httpContext, ApplicationDbContext DbContext, ISensitivePackage sensitive)
        {

            try
            {
                Microsoft.Extensions.Primitives.StringValues HttpAuthorizationValues;
                bool AnyAuthorizationHeader = httpContext.Request.Headers.TryGetValue("Authorization", out HttpAuthorizationValues);

                if (AnyAuthorizationHeader)
                {
                    string tokenRawValue = TokenManager.GetBearerTokenStringFromHttpHeader(HttpAuthorizationValues);
                    ITokenModel TokenDataBox = new TokenModel();
                    bool IsTokenVerifiedAndFilled = TokenManager.VerifyTokenAndRead(tokenRawValue, sensitive.GetSign(), ref TokenDataBox);

                    if (IsTokenVerifiedAndFilled)
                    {
                        var userFindTry = DbContext.Users.Where(u => u.UserID.Equals(TokenDataBox.UserIdentificator));
                        if (userFindTry.Any())
                        {
                            var user = userFindTry.First();
                            httpContext.User = SecurityManager.BuildPrincipal(user.UserID, user.Email);
                        }

                        else
                        {
                            httpContext.Response.Clear();
                            httpContext.Response.StatusCode = StatusCodes.Status401Unauthorized;
                            await httpContext.Response.WriteAsync("No User Found");
                            return;
                        }

                    }

                    else
                    {
                        httpContext.Response.Clear();
                        httpContext.Response.StatusCode = StatusCodes.Status401Unauthorized;
                        await httpContext.Response.WriteAsync("No Valid Token");
                        return;
                    }
                }

                await _next.Invoke(httpContext);
            }

            catch (Exception exc)
            {
                httpContext.Response.Clear();
                httpContext.Response.StatusCode = StatusCodes.Status400BadRequest;
                await httpContext.Response.WriteAsync(exc.Message);
                return;
            }
        }

       
    }

    // Extension method used to add the middleware to the HTTP request pipeline.
    public static class TokenAuthMiddlewareExtensions
    {
        public static IApplicationBuilder UseTokenAuthMiddleware(this IApplicationBuilder builder)
        {
            return builder.UseMiddleware<TokenAuthMiddleware>();
        }
    }
}
